/*******************************************************************************
 * Copyright (c) 2008 Exadel, Inc. and Red Hat, Inc.
 * Distributed under license by Red Hat, Inc. All rights reserved.
 * This program is made available under the terms of the
 * Eclipse Public License v1.0 which accompanies this distribution,
 * and is available at http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Exadel, Inc. and Red Hat, Inc. - initial API and implementation
 ******************************************************************************/
package org.hibernate.cfg.reveng;

import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.cfg.reveng.dialect.MetaDataDialect;
import org.hibernate.connection.ConnectionProvider;
import org.hibernate.exception.SQLExceptionConverter;
import org.hibernate.util.StringHelper;

/**
 * @author Vitali
 *
 */
public class JDBCTablesColumnsReader {

	private static final Log log = LogFactory.getLog(JDBCTablesColumnsReader.class);

	private MetaDataDialect metadataDialect;

	private final ConnectionProvider provider;
	private final SQLExceptionConverter sec;

	public JDBCTablesColumnsReader(MetaDataDialect dialect, ConnectionProvider provider,
			SQLExceptionConverter sec) {
		this.metadataDialect = dialect;
		this.provider = provider;
		this.sec = sec;
	}

	private boolean safeEquals(Object value, Object tf) {
		if (value == tf)
			return true;
		if (null == value)
			return false;
		return value.equals(tf);
	}

	private String quote(String columnName) {
		if (null == columnName) {
			return null;
		}
		if (!metadataDialect.needQuote(columnName)) {
			return columnName;
		}
		if (columnName.length() > 1 && columnName.charAt(0) == '`'
				&& columnName.charAt(columnName.length() - 1) == '`') {
			return columnName; // avoid double quoting
		}
		return "`" + columnName + "`";
	}

	private void collectTables(TablesColumnsCollector tcc, String catalog, String schema) {
		Iterator tableIterator = null;
		try {
			String tablesMask = null;
			tableIterator = metadataDialect.getTables(StringHelper.replace(catalog, ".*", "%"),
					StringHelper.replace(schema, ".*", "%"), StringHelper.replace(tablesMask, ".*",
							"%"));
			while (tableIterator.hasNext()) {
				Map tableRs = (Map) tableIterator.next();
				String tableName = (String) tableRs.get("TABLE_NAME");
				// String schemaName = (String) tableRs.get("TABLE_SCHEM");
				// String catalogName = (String) tableRs.get("TABLE_CAT");
				// String comment = (String) tableRs.get("REMARKS");
				String tableType = (String) tableRs.get("TABLE_TYPE");
				if (("TABLE".equals(tableType) || "VIEW".equals(tableType))) {
					log.debug("Adding table " + tableName + " of type " + tableType);
					tcc.addTableName(quote(tableName));
				} else {
					log.debug("Ignoring table " + tableName + " of type " + tableType);
				}
			}
		} finally {
			try {
				if (null != tableIterator) {
					metadataDialect.close(tableIterator);
				}
			} catch (Exception ignore) {
			}
		}
	}

	public void readDatabaseTables(TablesColumnsCollector tcc, String catalog, String schema) {
		try {
			ReverseEngineeringRuntimeInfo info = new ReverseEngineeringRuntimeInfo(provider, sec,
					null);
			metadataDialect.configure(info);
			tcc.init();
			collectTables(tcc, catalog, schema);
			tcc.adjust();
		} finally {
			metadataDialect.close();
		}
	}

	private void collectColumns(TablesColumnsCollector tcc, String catalog, String schema) {
		Iterator columnIterator = null;
		try {
			String tablesMask = null;
			String columnsMask = null;
			columnIterator = metadataDialect.getColumns(StringHelper.replace(catalog, ".*", "%"),
					StringHelper.replace(schema, ".*", "%"), StringHelper.replace(tablesMask, ".*",
							"%"), StringHelper.replace(columnsMask, ".*", "%"));
			while (columnIterator.hasNext()) {
				Map columnRs = (Map) columnIterator.next();
				String tableName = (String) columnRs.get("TABLE_NAME");
				String columnName = (String) columnRs.get("COLUMN_NAME");
				// String schemaName = (String) tableRs.get("TABLE_SCHEM");
				// String catalogName = (String) tableRs.get("TABLE_CAT");
				// String dataType = (String) tableRs.get("DATA_TYPE");
				log.debug("Adding column " + tableName + "." + columnName);
				tcc.addColumnName(quote(tableName), columnName);
			}
		} finally {
			try {
				if (null != columnIterator) {
					metadataDialect.close(columnIterator);
				}
			} catch (Exception ignore) {
			}
		}
	}

	public void readDatabaseColumns(TablesColumnsCollector tcc, String catalog, String schema) {
		try {
			ReverseEngineeringRuntimeInfo info = new ReverseEngineeringRuntimeInfo(provider, sec,
					null);
			metadataDialect.configure(info);
			tcc.init();
			collectColumns(tcc, catalog, schema);
			tcc.adjust();
		} finally {
			metadataDialect.close();
		}
	}

	public void readDatabaseTablesColumns(TablesColumnsCollector tcc, String catalog, String schema) {
		try {
			ReverseEngineeringRuntimeInfo info = new ReverseEngineeringRuntimeInfo(provider, sec,
					null);
			metadataDialect.configure(info);
			tcc.init();
			collectTables(tcc, catalog, schema);
			collectColumns(tcc, catalog, schema);
			tcc.adjust();
		} finally {
			metadataDialect.close();
		}
	}
}
